; SMM Share and Money Management
; Copyright (c) 2007-2010 Carsten Grohmann
;
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; as published by the Free Software Foundation; version 2 of the
; License.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
; 02110-1301, USA.

; HM NIS Edit Wizard helper defines
!define PRODUCT_NAME "Share and Money Management"
!define PRODUCT_NAME_SHORT "SMM"
!define PRODUCT_VERSION "0.16"
!define PRODUCT_PUBLISHER "Carsten Grohmann"
!define PRODUCT_WEB_SITE "http://www.carstengrohmann.de/smm.html"
!define PRODUCT_UNINST_KEY "Software\Microsoft\Windows\CurrentVersion\Uninstall\${PRODUCT_NAME}"
!define PRODUCT_UNINST_ROOT_KEY "HKLM"
!define PRODUCT_STARTMENU_REGVAL "NSIS:StartMenuDir"

!define ERR_INSTDIRNOTEMPTY "Das Installationsverzeichnis $INSTDIR konnte nicht gelscht werden, da sich noch einige Dateien darin befinden."

!define NOTE_COMPILEPYTHON "Die Quelldateien werden in Python-Byte-Code bersetzt."

Var PYTHON   ; path to python.exe, usually C:\Programe\Python25\python.exe
Var PYTHONW  ; path to pythonw.exe, usually C:\Programe\Python25\pythonw.exe

; MUI 1.67 compatible ------
!include "MUI.nsh"

; MUI Settings
!define MUI_ABORTWARNING
!define MUI_ICON "${NSISDIR}\Contrib\Graphics\Icons\modern-install.ico"
!define MUI_UNICON "${NSISDIR}\Contrib\Graphics\Icons\modern-uninstall.ico"

; Welcome page
!define MUI_WELCOMEPAGE_TITLE "Willkommen beim Installations-\r\nAssistenten fr ${PRODUCT_NAME_SHORT} ${PRODUCT_VERSION}"
!insertmacro MUI_PAGE_WELCOME
; License page
!define MUI_LICENSEPAGE_RADIOBUTTONS
!insertmacro MUI_PAGE_LICENSE "COPYING.GPL2.de"
; Directory page
!insertmacro MUI_PAGE_DIRECTORY
; Start menu page
var ICONS_GROUP
!define MUI_STARTMENUPAGE_NODISABLE
!define MUI_STARTMENUPAGE_DEFAULTFOLDER "${PRODUCT_NAME}"
!define MUI_STARTMENUPAGE_REGISTRY_ROOT "${PRODUCT_UNINST_ROOT_KEY}"
!define MUI_STARTMENUPAGE_REGISTRY_KEY "${PRODUCT_UNINST_KEY}"
!define MUI_STARTMENUPAGE_REGISTRY_VALUENAME "${PRODUCT_STARTMENU_REGVAL}"
!insertmacro MUI_PAGE_STARTMENU Application $ICONS_GROUP
; Instfiles page
!insertmacro MUI_PAGE_INSTFILES
; Finish page
!define MUI_FINISHPAGE_TITLE "Die Installation von ${PRODUCT_NAME_SHORT} ${PRODUCT_VERSION} wird abgeschlossen"
!define MUI_FINISHPAGE_LINK "Webseite ${PRODUCT_NAME}"
!define MUI_FINISHPAGE_LINK_LOCATION "${PRODUCT_WEB_SITE}"
!define MUI_FINISHPAGE_SHOWREADME "$INSTDIR\ChangeLog.txt"
!define MUI_FINISHPAGE_SHOWREADME_TEXT "nderungshistorie"
!insertmacro MUI_PAGE_FINISH

; Uninstaller pages
!insertmacro MUI_UNPAGE_INSTFILES

; Language files
!insertmacro MUI_LANGUAGE "German"

; MUI end ------

Name "${PRODUCT_NAME} ${PRODUCT_VERSION}"
OutFile "smm-${PRODUCT_VERSION}.win32.exe"
InstallDir "$PROGRAMFILES\${PRODUCT_NAME}"
ShowInstDetails hide
ShowUnInstDetails hide
XPStyle on
InstProgressFlags smooth
AutoCloseWindow false

; Request admin privilages caused by writing HKLM
RequestExecutionLevel admin

Function .onInit

 ; prevent multiple installer instances
  System::Call 'kernel32::CreateMutexA(i 0, i 0, t "SMMINSTPROG") i .r1 ?e'
  Pop $R0

  StrCmp $R0 0 NotRunning
    MessageBox MB_OK|MB_ICONEXCLAMATION "Es luft schon eine Instanz des Installers"
    Abort
NotRunning:
  
  ; Check to see if already installed
  ReadRegStr $R0 ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "UninstallString"
  IfFileExists $R0 +1 ContinueCheckPython
  MessageBox MB_OK|MB_ICONEXCLAMATION "${PRODUCT_NAME} ist bereits installiert.$\r$\nBitte deinstallieren Sie es, bevor Sie diese Installation erneut starten."
  Abort

  ; Check for python 2.5
  ContinueCheckPython:
  ReadRegStr $0 HKLM "SOFTWARE\Python\PythonCore\2.5\InstallPath" ""
  StrCpy $PYTHON "$0\python.exe"
  StrCpy $PYTHONW "$0\pythonw.exe"
  StrCmp $0 "" 0 ContinueInstallation
  MessageBox MB_OK|MB_ICONEXCLAMATION "Es wurde kein Eintrag fr Python 2.5 in der Registry gefunden.$\r$\nBitte installieren Sie Python 2.5, bevor sie ${PRODUCT_NAME} installieren."
    Abort

  ContinueInstallation:

FunctionEnd

; It can sometimes be useful to convert DOS style line endings of text
; files, such as the README or CHANGELOG, because some windows apps such
; as notepad can't display files UNIX style line endings properly. The
; function below converts line endings to DOS style directly from within
; NSIS without the need for any external plugin or external converter.
; Source: http://nsis.sourceforge.net/UNIX_to_DOS_text_file_format_conversion_script
Function unix2dos
    ; strips all CRs
    ; and then converts all LFs into CRLFs
    ; (this is roughly equivalent to "cat file | dos2unix | unix2dos")
    ;
    ; usage:
    ;    Push "infile"
    ;    Push "outfile"
    ;    Call unix2dos
    ;
    ; beware that this function destroys $0 $1 $2

    ClearErrors

    Pop $2
    FileOpen $1 $2 w

    Pop $2
    FileOpen $0 $2 r

    Push $2 ; save name for deleting

    IfErrors unix2dos_done

    ; $0 = file input (opened for reading)
    ; $1 = file output (opened for writing)

unix2dos_loop:
    ; read a byte (stored in $2)
    FileReadByte $0 $2
    IfErrors unix2dos_done ; EOL
    ; skip CR
    StrCmp $2 13 unix2dos_loop
    ; if LF write an extra CR
    StrCmp $2 10 unix2dos_cr unix2dos_write

unix2dos_cr:
    FileWriteByte $1 13

unix2dos_write:
    ; write byte
    FileWriteByte $1 $2
    ; read next byte
    Goto unix2dos_loop

unix2dos_done:

    ; close files
    FileClose $0
    FileClose $1

    ; delete original
    Pop $0
    Delete $0
FunctionEnd

Section "Install Application" SEC01
  SetOutPath "$INSTDIR"
  SetOverwrite ifnewer
  
  File ChangeLog
  Push "ChangeLog"
  Push "ChangeLog.txt"
  Call unix2dos

  File COPYING.GPL2
  Push "COPYING.GPL2"
  Push "COPYING.GPL2.txt"
  Call unix2dos

  File COPYING.GPL2.de
  Push "COPYING.GPL2.de"
  Push "COPYING.GPL2.de.txt"
  Call unix2dos
  
  File README
  Push "README"
  PUSH "README.txt"
  Call unix2dos

  File "smmgui.py"
  File "smmgui.pyw"
  File "smmshell.py"
  File "BeautifulSoup.py"
  
  CreateDirectory "$INSTDIR\doc"
  SetOutPath "$INSTDIR\doc"
  File /r doc\*.*

  CreateDirectory "$INSTDIR\gui"
  SetOutPath "$INSTDIR\gui"
  File /r gui\*.py

  CreateDirectory "$INSTDIR\mako"
  SetOutPath "$INSTDIR\mako"
  File /r mako\*.py

  CreateDirectory "$INSTDIR\smm"
  SetOutPath "$INSTDIR\smm"
  File /r /x vwd.py /x onvista.py smm\*.py

  CreateDirectory "$INSTDIR\storm"
  SetOutPath "$INSTDIR\storm"
  File /r storm\*.py
  
; Shortcuts
  ; select "All Users"
  SetShellVarContext all
  
  ; Reset path used for working directory
  SetOutPath "$INSTDIR"
  
  !insertmacro MUI_STARTMENU_WRITE_BEGIN Application
  CreateDirectory "$SMPROGRAMS\$ICONS_GROUP"
  CreateShortCut "$SMPROGRAMS\$ICONS_GROUP\SMM GUI.lnk" "$INSTDIR\smmgui.pyw"
  CreateShortCut "$SMPROGRAMS\$ICONS_GROUP\SMM Shell (Entwicklung).lnk" "$INSTDIR\smmshell.py"
  CreateShortCut "$SMPROGRAMS\$ICONS_GROUP\README.lnk" "$INSTDIR\README.txt"
  CreateShortCut "$SMPROGRAMS\$ICONS_GROUP\ChangeLog.lnk" "$INSTDIR\ChangeLog.txt"
  CreateShortCut "$SMPROGRAMS\$ICONS_GROUP\Lizenz GPL2 (Deutsch).lnk" "$INSTDIR\COPYING.GPL2.de.txt"
  CreateShortCut "$SMPROGRAMS\$ICONS_GROUP\Lizenz GPL2 (Englisch).lnk" "$INSTDIR\COPYING.GPL2.txt"
  !insertmacro MUI_STARTMENU_WRITE_END
SectionEnd

Section -AdditionalIcons
  SetOutPath $INSTDIR
  !insertmacro MUI_STARTMENU_WRITE_BEGIN Application
  WriteIniStr "$INSTDIR\${PRODUCT_NAME}.url" "InternetShortcut" "URL" "${PRODUCT_WEB_SITE}"
  CreateShortCut "$SMPROGRAMS\$ICONS_GROUP\Website.lnk" "$INSTDIR\${PRODUCT_NAME}.url"
  CreateShortCut "$SMPROGRAMS\$ICONS_GROUP\Uninstall.lnk" "$INSTDIR\uninst.exe"
  !insertmacro MUI_STARTMENU_WRITE_END
SectionEnd

Section -Post
  WriteUninstaller "$INSTDIR\uninst.exe"
  WriteRegStr   ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "DisplayName" "$(^Name)"
  WriteRegStr   ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "UninstallString" "$INSTDIR\uninst.exe"
  WriteRegStr   ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "DisplayVersion" "${PRODUCT_VERSION}"
  WriteRegStr   ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "URLInfoAbout" "${PRODUCT_WEB_SITE}"
  WriteRegStr   ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "Publisher" "${PRODUCT_PUBLISHER}"
  WriteRegDWORD ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "NoModify" 0x00000001
  WriteRegDWORD ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}" "NoRepair" 0x00000001
  
  ; compiling python files
  DetailPrint "${NOTE_COMPILEPYTHON}"
  nsExec::ExecToLog '"$PYTHON" -c "import sys,compileall; compileall.compile_dir(sys.argv[1])" "$INSTDIR"'
  nsExec::ExecToLog '"$PYTHON" -O -c "import sys,compileall; compileall.compile_dir(sys.argv[1])" "$INSTDIR"'
SectionEnd


;Function un.onUninstSuccess
;  HideWindow
;  MessageBox MB_ICONINFORMATION|MB_OK "$(^Name) wurde erfolgreich deinstalliert."
;FunctionEnd

Function un.onInit
  MessageBox MB_ICONQUESTION|MB_YESNO|MB_DEFBUTTON2 "Mchten Sie $(^Name) und alle seinen Komponenten deinstallieren?" IDYES +2
  Abort
FunctionEnd

Section Uninstall
  !insertmacro MUI_STARTMENU_GETFOLDER "Application" $ICONS_GROUP
  
  ; select "All Users"
  SetShellVarContext all

  ; remove single files / directories
  Delete $INSTDIR\BeautifulSoup.py*
  Delete $INSTDIR\ChangeLog.txt
  Delete $INSTDIR\COPYING.GPL2.txt
  Delete $INSTDIR\COPYING.GPL2.de.txt
  Delete $INSTDIR\README.txt
  Delete $INSTDIR\setup.py*
  Delete $INSTDIR\smmgui.py*
  Delete $INSTDIR\smmgui.pyw
  Delete $INSTDIR\smmshell.py*
  RMDir /r $INSTDIR\doc
  RMDir /r $INSTDIR\gui
  RMDir /r $INSTDIR\mako
  RMDir /r $INSTDIR\smm
  RMDir /r $INSTDIR\storm

  Delete "$INSTDIR\${PRODUCT_NAME}.url"
  Delete "$INSTDIR\uninst.exe"

  Delete "$SMPROGRAMS\$ICONS_GROUP\Uninstall.lnk"
  Delete "$SMPROGRAMS\$ICONS_GROUP\Website.lnk"
  Delete "$SMPROGRAMS\$ICONS_GROUP\SMM Shell (Entwicklung).lnk"
  Delete "$SMPROGRAMS\$ICONS_GROUP\ChangeLog.lnk"
  Delete "$SMPROGRAMS\$ICONS_GROUP\Lizenz GPL2 (Englisch).lnk"
  Delete "$SMPROGRAMS\$ICONS_GROUP\Lizenz GPL2 (Deutsch).lnk"
  Delete "$SMPROGRAMS\$ICONS_GROUP\README.lnk"
  Delete "$SMPROGRAMS\$ICONS_GROUP\SMM GUI.lnk"

  RMDir "$SMPROGRAMS\$ICONS_GROUP"
  RMDir "$INSTDIR\storm\databases"
  RMDir "$INSTDIR\storm"
  RMDir "$INSTDIR\smm\quote"
  RMDir "$INSTDIR\smm"
  RMDir "$INSTDIR\mako"
  RMDir "$INSTDIR\gui"
  RMDir "$INSTDIR\doc"
  
  ; try to delete installation directory
  ClearErrors
  RMDir "$INSTDIR"
  IfErrors 0 +3
    MessageBox MB_OK "${ERR_INSTDIRNOTEMPTY}"
    DetailPrint "${ERR_INSTDIRNOTEMPTY}"

  DeleteRegKey ${PRODUCT_UNINST_ROOT_KEY} "${PRODUCT_UNINST_KEY}"
SectionEnd